/*
 * $Id: alist.c,v 1.10 1998/05/14 18:49:41 masaki Exp $
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <netdb.h>
#include <signal.h>
#include <fcntl.h>
#include <errno.h>
#include <array.h>
#include <New.h>
#include <hash.h>
#include <mrt.h>
#include <io.h>
#include <bgp_proto.h>
#include <aspath.h>
#include <bgp.h>

static LINKED_LIST *access_list[MAX_ALIST];

int
add_access_list (u_int num, prefix_t *prefix, int permit, int refine) {

   condition_t *condition;

   if (num >= MAX_ALIST)
	 return (-1);
   if (access_list[num] == NULL) {
      access_list[num] = LL_Create (0);
   }
   condition = New (condition_t);
   condition->permit = permit;
   condition->refine = refine;
   /* expects Ref_Prefix can handle even a case of prefix == NULL */
   condition->prefix = Ref_Prefix (prefix);
   LL_Add (access_list[num], condition);
   return (1);
}

int 
remove_access_list (u_int num, prefix_t *prefix, int permit, int refine) {
  condition_t *condition;

  if (num >= MAX_ALIST)
    return (-1);
  if (access_list[num] == NULL) {
    return (-1);
  }

   LL_Iterate (access_list[num], condition) {
     /* prefix == NULL if 'all' is specified in place of prefix */
     /* because family is unknown in case of 'all' */
     if (((prefix == NULL && condition->prefix == NULL) ||
          (prefix && condition->prefix && 
		prefix_compare (prefix, condition->prefix))) &&
         (permit == condition->permit) &&
         (refine == condition->refine)) {
       Deref_Prefix (condition->prefix);
       LL_Remove (access_list[num], condition);
       return (1);
     }
   }

   return (-1);
}


int
del_access_list (u_int num) {

   if (num >= MAX_ALIST)
	 return (-1);
    if (access_list[num] == NULL) {
        return (-1);
    }
    Delete (access_list[num]);
    access_list[num] = NULL;
    return (1);
}


/*
 * return 1 if permit, 0 otherwise
 */
int apply_access_list (u_int num, prefix_t *prefix) {

   condition_t *condition;

   if (num >= MAX_ALIST)
     return (0);
   if (num == 0)
      return (1); /* cisco feature */
   if (access_list[num] == NULL) {
      /* I'm not sure how cisco works for undefined access lists */
     return (0); /* assuming deny for now */
   }

   LL_Iterate (access_list[num], condition) {

      if (condition->prefix == NULL) /* all */
	  return (condition->permit);

      if ((prefix->family == condition->prefix->family) && (
          (condition->refine && (prefix->bitlen > condition->prefix->bitlen)) ||
          (!condition->refine && 
              (prefix->bitlen == condition->prefix->bitlen))) &&
           comp_with_mask (prefix_tochar (prefix), 
                           prefix_tochar (condition->prefix),
                           condition->prefix->bitlen))
         return (condition->permit);
   }
   return (0);
}



char *access_list_toa () {
  condition_t *condition;
  char *cp, *tag, *refine;
  char *tmp;
  int i;
  
  tmp = malloc (1024*4);
  cp = tmp;

  for (i=0; i < MAX_ALIST; i++) {
    if (access_list[i] == NULL) continue;
    LL_Iterate (access_list[i], condition) {
      if (condition->permit) 
	tag = "permit";
      else
	tag = "deny";
      if (condition->refine == 1) 
	refine = "refine";
      else
	refine = "exact";

      if (condition->prefix == NULL)
          sprintf (cp, "access-list %d %s all\r\n", i, tag);
      else
          sprintf (cp, "access-list %d %s %s %s\r\n", i, tag, 
	           prefix_toax (condition->prefix), refine);
      cp += strlen (cp);


    }
  }

  return (tmp);
}
