/*
 * $Id: main.c,v 1.9 1998/05/05 16:18:46 labovit Exp $
 */

#include <sys/types.h> 
#include <fcntl.h>
#include <mrt.h>
#include <select.h>
#include <config_file.h>
#include "salserver.h"

trace_t			*default_trace;
salserver_t		SALSERVER;
char thisInterfaceVersion[4] = SAL_SERVER_INTERFACE_VERSION;

void die () {
  exit (0);
}


void sal_broken_pipe () {
  trace (NORM, default_trace, "***** GOT SIGPIPE ********\n");
}

void main(int argc, char *argv[])
{
  char *name = (char *) argv[0];
  int daemon = 0;
  int trace_set = 0;
  extern char *optarg;	/* getopt stuff */
  extern int optind;	/* getopt stuff */
  char c;
  char *port = "salserver";
  char *key = NULL;	/* password for UII */
  int errors = 0;

  char *usage ="Usage: %s [-f config_file] [-p uii_port ] [-v]\n";

  /* defaults */
  default_trace = New_Trace ();
  SALSERVER.udp_port = 8086;
  SALSERVER.tcp_port = 9000;
  SALSERVER.config_file = "/etc/salserver.conf";
  SALSERVER.ll_sal_connections = LL_Create (NULL);
  SALSERVER.ll_multiserver = LL_Create (NULL);
  SALSERVER.ll_archives = LL_Create (NULL);
  SALSERVER.ll_channels = LL_Create (NULL);
  SALSERVER.max_data_size = 4096*50;  /* too big ! */
  SALSERVER.max_connections = 60;
  SALSERVER.max_idle = 120;
  SALSERVER.key = 123456;
  SALSERVER.dir = "/var/tmp/";

  /* init MRT service daemons */
  init_trace (name, daemon);
  init_mrt (default_trace);
  init_config (default_trace);
  init_mrt_reboot (argc, argv);

  SALSERVER.multiserver_retry_timer = (mtimer_t *)
    New_Timer (multiserver_retry, 60*5, "MultiServer Retry timer", NULL);

  SALSERVER.connection_schedule = (schedule_t *) 
    New_Schedule ("Connection", default_trace);

  while ((c = getopt (argc, argv, "hnvf:p:k:c:s:")) != -1) 
     switch (c) {
     case 'v': /* verbose */
       set_trace (default_trace, TRACE_FLAGS, NORM | TR_TRACE | TR_ERROR, 
		  TRACE_LOGFILE, "stdout",
		  NULL);
	trace_set = 1;
	daemon = 0;
       break;
     case 'f': /* config file */
       SALSERVER.config_file = strdup (optarg);
       break;
     case 'p': /* uii port number */
       port = optarg;
       break;
     case 'k': /* key (password for UII) */
       key = optarg;
       break;
     case 'c': /* community */
       break;
     case 'h':
     default:
       errors++;
       break;
     }

   if (errors) {
     fprintf(stderr, usage, name);
     printf ("\nMRT %s compiled on %s\n\n",
	     MRT_VERSION, __DATE__);
     exit (1);
   }
  
 

  UII->initial_state = 0;
  set_uii (UII, UII_PROMPT, 0, "password> ");
  set_uii (UII, UII_PROMPT, 1, "SalServer> ");
  set_uii (UII, UII_PROMPT, UII_CONFIG, "Config> ");
  set_uii (UII, UII_ACCESS_LIST, SALSERVER.telnet_access_list);

  uii_add_command2 (UII_UNPREV, COMMAND_MATCH_FIRST, "",
		    (void *) uii_check_passwd,
		    "Password");
  uii_add_command2 (UII_NORMAL, COMMAND_NORM, "show connections", 
		    (void *) schedule_show_connections,
		    "Show client publishers and receviers");
  uii_add_command2 (UII_NORMAL, COMMAND_NORM, "show channels", 
		    (void *) schedule_show_channels,
		    "Show channels currently published");
  uii_add_command2 (UII_NORMAL, COMMAND_NORM, "show multiservers", 
		    (void *) show_multiservers,
		    "Show status of peer Salserver daemons");
  uii_add_command2 (UII_NORMAL, COMMAND_NORM,
		    "show config", show_config, "Display current configuration");
  uii_add_command2 (UII_NORMAL, COMMAND_NODISPLAY, "die", (void *) die, NULL);
  uii_add_command2 (UII_NORMAL, COMMAND_NORM, "config", start_config, 
		    "Configure Salserver options");
  uii_add_command2 (UII_NORMAL, COMMAND_NORM, "write", config_write, 
		    "Write configuration to disk");
  uii_add_command2 (UII_NORMAL, COMMAND_NORM, "close channel", 
		    (void *) close_salserver_connection, 
		    "Close a Salserver connection");
  uii_add_command2 (UII_NORMAL, COMMAND_NORM, "reboot", (void *) mrt_reboot,
		    "Reboot IRRd");
  uii_add_command2 (UII_CONFIG, COMMAND_NORM, "database %s", (void *) config_database,
		    "Directory where archived files are stored");
  uii_add_command2 (UII_CONFIG, COMMAND_NORM, "multiserver", 
		    (void *) config_multiserver,
		    "Remote mirror SalServer servers");
  uii_add_command2 (UII_CONFIG, COMMAND_NORM, "archive %s %s", (void *) config_archive,
		    "Channels to archive");


  if (config_from_file (default_trace, SALSERVER.config_file) < 1) {
    config_create_default ();
  }

  start_connection_thread ();
  listen_uii2 ("salserver");

  Timer_Turn_ON ((mtimer_t *) SALSERVER.multiserver_retry_timer);

  listen_udp_port ();
  if (listen_tcp_port () == -1) {
    printf ("Could not bind to TCP port -- another server running?\n");
    exit (0);
  }

  mrt_main_loop ();
  /* NOT REACHED */

}






